@@ -6,14 +6,14 @@
  */
 
 #include "ixion/formula.hpp"
-#include "ixion/formula_lexer.hpp"
 #include "ixion/formula_name_resolver.hpp"
-#include "ixion/formula_parser.hpp"
 #include "ixion/formula_function_opcode.hpp"
 #include "ixion/cell.hpp"
 #include "ixion/cell_listener_tracker.hpp"
 #include "ixion/types.hpp"
 
+#include "formula_lexer.hpp"
+#include "formula_parser.hpp"
 #include "function_objects.hpp"
 #include "formula_functions.hpp"
 #include "depends_tracker.hpp"
@@ -21,6 +21,7 @@
 #define DEBUG_FORMULA_API 0
 
 #include <sstream>
+#include <algorithm>
 
 #if DEBUG_FORMULA_API
 #include <iostream>
@@ -147,12 +148,24 @@ void print_formula_tokens(
 
 namespace {
 
+bool is_volatile(formula_function_t func)
+{
+    switch (func)
+    {
+        case formula_function_t::func_now:
+            return true;
+        default:
+            ;
+    }
+    return false;
+}
+
 bool has_volatile(const formula_tokens_t& tokens)
 {
     formula_tokens_t::const_iterator i = tokens.begin(), iend = tokens.end();
     for (; i != iend; ++i)
     {
-        const formula_token_base& t = **i;
+        const formula_token& t = **i;
         if (t.get_opcode() != fop_function)
             continue;
 
@@ -172,8 +185,7 @@ void register_formula_cell(iface::formula_model_access& cxt, const abs_address_t
         // Not a formula cell. Bail out.
         return;
 
-    std::vector<const formula_token_base*> ref_tokens;
-    cell->get_ref_tokens(cxt, pos, ref_tokens);
+    std::vector<const formula_token*> ref_tokens = cell->get_ref_tokens(cxt, pos);
     std::for_each(ref_tokens.begin(), ref_tokens.end(),
              formula_cell_listener_handler(cxt,
                  pos, formula_cell_listener_handler::mode_add));
@@ -199,8 +211,7 @@ void unregister_formula_cell(iface::formula_model_access& cxt, const abs_address
     // Go through all its existing references, and remove
     // itself as their listener.  This step is important
     // especially during partial re-calculation.
-    std::vector<const formula_token_base*> ref_tokens;
-    fcell->get_ref_tokens(cxt, pos, ref_tokens);
+    std::vector<const formula_token*> ref_tokens = fcell->get_ref_tokens(cxt, pos);
     for_each(ref_tokens.begin(), ref_tokens.end(),
              formula_cell_listener_handler(cxt,
                  pos, formula_cell_listener_handler::mode_remove));
