@@ -7,6 +7,7 @@
 
 #include "formula_interpreter.hpp"
 #include "formula_functions.hpp"
+#include "concrete_formula_tokens.hpp"
 
 #include "ixion/cell.hpp"
 #include "ixion/global.hpp"
@@ -42,9 +43,8 @@ opcode_token paren_close = opcode_token(fop_close);
 formula_interpreter::formula_interpreter(const formula_cell* cell, iface::formula_model_access& cxt) :
     m_parent_cell(cell),
     m_context(cxt),
-    mp_handler(NULL),
     m_stack(cxt),
-    m_error(fe_no_error)
+    m_error(formula_error_t::no_error)
 {
 }
 
@@ -59,7 +59,7 @@ void formula_interpreter::set_origin(const abs_address_t& pos)
 
 bool formula_interpreter::interpret()
 {
-    mp_handler = m_context.get_session_handler();
+    mp_handler = m_context.create_session_handler();
     if (mp_handler)
         mp_handler->begin_cell_interpret(m_pos);
 
@@ -76,7 +76,7 @@ bool formula_interpreter::interpret()
         }
 
         m_cur_token_itr = m_tokens.begin();
-        m_error = fe_no_error;
+        m_error = formula_error_t::no_error;
         m_result.reset();
 
         expression();
@@ -91,6 +91,9 @@ bool formula_interpreter::interpret()
 #if DEBUG_FORMULA_INTERPRETER
         __IXION_DEBUG_OUT__ << "interpretation successfully finished" << endl;
 #endif
+        if (mp_handler)
+            mp_handler->end_cell_interpret();
+
         return true;
     }
     catch (const invalid_expression& e)
@@ -98,7 +101,7 @@ bool formula_interpreter::interpret()
         if (mp_handler)
             mp_handler->set_invalid_expression(e.what());
 
-        m_error = fe_invalid_expression;
+        m_error = formula_error_t::invalid_expression;
     }
     catch (const formula_error& e)
     {
@@ -110,6 +113,10 @@ bool formula_interpreter::interpret()
 
         m_error = e.get_error();
     }
+
+    if (mp_handler)
+        mp_handler->end_cell_interpret();
+
     return false;
 }
 
@@ -140,7 +147,7 @@ void formula_interpreter::init_tokens()
     formula_tokens_t::const_iterator itr = orig_tokens->begin(), itr_end = orig_tokens->end();
     for (; itr != itr_end; ++itr)
     {
-        const formula_token_base* p = &(**itr);
+        const formula_token* p = &(**itr);
         assert(p);
         if (p->get_opcode() == fop_named_expression)
         {
@@ -168,9 +175,7 @@ void get_result_from_cell(const iface::formula_model_access& cxt, const abs_addr
             if (!fcell)
                 return;
 
-            const formula_result* fres = fcell->get_result_cache();
-            if (fres)
-                res = *fres;
+            res = fcell->get_result_cache();
         }
         break;
         case celltype_t::numeric:
@@ -241,7 +246,7 @@ void formula_interpreter::expand_named_expression(
     formula_tokens_t::const_iterator itr = expr_tokens->begin(), itr_end = expr_tokens->end();
     for (; itr != itr_end; ++itr)
     {
-        const formula_token_base& t = **itr;
+        const formula_token& t = **itr;
         if (t.get_opcode() == fop_named_expression)
         {
             string expr_name = t.get_name();
@@ -270,13 +275,13 @@ void formula_interpreter::next()
     ++m_cur_token_itr;
 }
 
-const formula_token_base& formula_interpreter::token() const
+const formula_token& formula_interpreter::token() const
 {
     assert(m_cur_token_itr != m_end_token_pos);
     return *(*m_cur_token_itr);
 }
 
-const formula_token_base& formula_interpreter::next_token()
+const formula_token& formula_interpreter::next_token()
 {
     next();
     if (!has_token())
@@ -351,29 +356,27 @@ bool pop_stack_value_or_string(const iface::formula_model_access& cxt,
                 {
                     const formula_cell* fc = cxt.get_formula_cell(addr);
                     assert(fc);
-                    const formula_result* res = fc->get_result_cache();
-                    if (!res)
-                        return false;
+                    const formula_result& res = fc->get_result_cache();
 
-                    switch (res->get_type())
+                    switch (res.get_type())
                     {
-                        case formula_result::rt_value:
+                        case formula_result::result_type::value:
                         {
                             vt = stack_value_t::value;
-                            val = res->get_value();
+                            val = res.get_value();
                             return true;
                         }
-                        case formula_result::rt_string:
+                        case formula_result::result_type::string:
                         {
                             vt = stack_value_t::string;
-                            string_id_t strid = res->get_string();
+                            string_id_t strid = res.get_string();
                             const string* ps = cxt.get_string(strid);
                             if (!ps)
                                 return false;
                             str = *ps;
                             return true;
                         }
-                        case formula_result::rt_error:
+                        case formula_result::result_type::error:
                         default:
                             return false;
                     }
@@ -429,7 +432,7 @@ void compare_strings(value_stack_t& vs, fopcode_t oc, const std::string& str1, c
     {
         case fop_plus:
         case fop_minus:
-            throw formula_error(fe_invalid_expression);
+            throw formula_error(formula_error_t::invalid_expression);
         case fop_equal:
             vs.push_value(str1 == str2);
         break;
@@ -462,7 +465,7 @@ void compare_value_to_string(
     {
         case fop_plus:
         case fop_minus:
-            throw formula_error(fe_invalid_expression);
+            throw formula_error(formula_error_t::invalid_expression);
         case fop_equal:
             vs.push_value(false);
         break;
@@ -491,7 +494,7 @@ void compare_string_to_value(
     {
         case fop_plus:
         case fop_minus:
-            throw formula_error(fe_invalid_expression);
+            throw formula_error(formula_error_t::invalid_expression);
         case fop_equal:
             vs.push_value(false);
         break;
@@ -533,7 +536,7 @@ void formula_interpreter::expression()
 
         stack_value_t vt;
         if (!pop_stack_value_or_string(m_context, m_stack, vt, val1, str1))
-            throw formula_error(fe_general_error);
+            throw formula_error(formula_error_t::general_error);
         is_val1 = vt == stack_value_t::value;
 
         if (mp_handler)
@@ -543,7 +546,7 @@ void formula_interpreter::expression()
         term();
 
         if (!pop_stack_value_or_string(m_context, m_stack, vt, val2, str2))
-            throw formula_error(fe_general_error);
+            throw formula_error(formula_error_t::general_error);
         is_val2 = vt == stack_value_t::value;
 
         if (is_val1)
@@ -606,7 +609,7 @@ void formula_interpreter::term()
             term();
             double val2 = m_stack.pop_value();
             if (val2 == 0.0)
-                throw formula_error(fe_division_by_zero);
+                throw formula_error(formula_error_t::division_by_zero);
             m_stack.push_value(val/val2);
             return;
         }
@@ -630,7 +633,7 @@ void formula_interpreter::factor()
             __IXION_DEBUG_OUT__ << "named expression encountered in factor" << endl;
 #endif
             // All named expressions are supposed to be expanded prior to interpretation.
-            throw formula_error(fe_general_error);
+            throw formula_error(formula_error_t::general_error);
         case fop_value:
             constant();
             return;
@@ -693,7 +696,7 @@ void formula_interpreter::single_ref()
     if (abs_addr == m_pos)
     {
         // self-referencing is not permitted.
-        throw formula_error(fe_ref_result_not_available);
+        throw formula_error(formula_error_t::ref_result_not_available);
     }
 
     m_stack.push_single_ref(abs_addr);
@@ -720,7 +723,7 @@ void formula_interpreter::range_ref()
     if (abs_range.contains(m_pos))
     {
         // Referenced range contains the address of this cell.  Not good.
-        throw formula_error(fe_ref_result_not_available);
+        throw formula_error(formula_error_t::ref_result_not_available);
     }
 
     m_stack.push_range_ref(abs_range);
@@ -735,7 +738,7 @@ void formula_interpreter::table_ref()
 #if DEBUG_FORMULA_INTERPRETER
         __IXION_DEBUG_OUT__ << "formula_interpreter::table_ref: failed to get a table_handler instance." << endl;
 #endif
-        throw formula_error(fe_ref_result_not_available);
+        throw formula_error(formula_error_t::ref_result_not_available);
     }
 
     table_t table = token().get_table_ref();
