@@ -17,8 +17,6 @@
 #include <sstream>
 #include <unordered_set>
 
-#include <boost/noncopyable.hpp>
-
 namespace ixion {
 
 class formula_cell;
@@ -43,12 +41,16 @@ class session_handler;
  * of stack values may be more than one when the function may take more than
  * one argument.</p>
  */
-class formula_interpreter : public ::boost::noncopyable
+class formula_interpreter
 {
     typedef std::unordered_set< ::std::string> name_set;
 
 public:
-    typedef ::std::vector<const formula_token_base*> local_tokens_type;
+    typedef ::std::vector<const formula_token*> local_tokens_type;
+
+    formula_interpreter() = delete;
+    formula_interpreter(const formula_interpreter&) = delete;
+    formula_interpreter& operator= (formula_interpreter) = delete;
 
     formula_interpreter(const formula_cell* cell, iface::formula_model_access& cxt);
     ~formula_interpreter();
@@ -72,8 +74,8 @@ private:
 
     bool has_token() const;
     void next();
-    const formula_token_base& token() const;
-    const formula_token_base& next_token();
+    const formula_token& token() const;
+    const formula_token& next_token();
 
     // The following methods are handlers.  In each handler, the initial
     // position is always set to the first unprocessed token.  Each handler is
@@ -94,7 +96,7 @@ private:
 private:
     const formula_cell* m_parent_cell;
     iface::formula_model_access& m_context;
-    iface::session_handler* mp_handler;
+    std::unique_ptr<iface::session_handler> mp_handler;
     abs_address_t m_pos;
 
     value_stack_t m_stack;
