@@ -5,11 +5,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include "ixion/formula_parser.hpp"
+#include "formula_parser.hpp"
 #include "ixion/formula_name_resolver.hpp"
 #include "ixion/interface/formula_model_access.hpp"
 
 #include "formula_functions.hpp"
+#include "concrete_formula_tokens.hpp"
 
 #include <iostream>
 #include <sstream>
@@ -29,14 +30,14 @@ public:
         general_error(msg) {}
 };
 
-class formula_token_printer : public unary_function<formula_token_base, void>
+class formula_token_printer : public unary_function<formula_token, void>
 {
 public:
     formula_token_printer()
     {
     }
 
-    void operator() (const formula_token_base& token) const
+    void operator() (const formula_token& token) const
     {
         fopcode_t oc = token.get_opcode();
         ostringstream os;
@@ -145,29 +146,29 @@ void formula_parser::parse()
             lexer_opcode_t oc = t.get_opcode();
             switch (oc)
             {
-                case op_open:
-                case op_close:
-                case op_plus:
-                case op_minus:
-                case op_multiply:
-                case op_equal:
-                case op_divide:
-                case op_sep:
+                case lexer_opcode_t::open:
+                case lexer_opcode_t::close:
+                case lexer_opcode_t::plus:
+                case lexer_opcode_t::minus:
+                case lexer_opcode_t::multiply:
+                case lexer_opcode_t::equal:
+                case lexer_opcode_t::divide:
+                case lexer_opcode_t::sep:
                     primitive(oc);
                     break;
-                case op_name:
+                case lexer_opcode_t::name:
                     name(t);
                     break;
-                case op_string:
+                case lexer_opcode_t::string:
                     literal(t);
                     break;
-                case op_value:
+                case lexer_opcode_t::value:
                     value(t);
                     break;
-                case op_less:
+                case lexer_opcode_t::less:
                     less(t);
                     break;
-                case op_greater:
+                case lexer_opcode_t::greater:
                     greater(t);
                     break;
                 default:
@@ -209,28 +210,28 @@ void formula_parser::primitive(lexer_opcode_t oc)
     fopcode_t foc = fop_unknown;
     switch (oc)
     {
-        case op_close:
+        case lexer_opcode_t::close:
             foc = fop_close;
             break;
-        case op_divide:
+        case lexer_opcode_t::divide:
             foc = fop_divide;
             break;
-        case op_minus:
+        case lexer_opcode_t::minus:
             foc = fop_minus;
             break;
-        case op_multiply:
+        case lexer_opcode_t::multiply:
             foc = fop_multiply;
             break;
-        case op_equal:
+        case lexer_opcode_t::equal:
             foc = fop_equal;
             break;
-        case op_open:
+        case lexer_opcode_t::open:
             foc = fop_open;
             break;
-        case op_plus:
+        case lexer_opcode_t::plus:
             foc = fop_plus;
             break;
-        case op_sep:
+        case lexer_opcode_t::sep:
             foc = fop_sep;
             break;
         default:
@@ -243,20 +244,20 @@ void formula_parser::name(const lexer_token_base& t)
 {
     mem_str_buf name = t.get_string();
 
-    formula_name_type fn = m_resolver.resolve(name.get(), name.size(), m_pos);
+    formula_name_t fn = m_resolver.resolve(name.get(), name.size(), m_pos);
 #if DEBUG_FORMULA_PARSER
             __IXION_DEBUG_OUT__ << "name = '" << name << "' - " << fn.to_string() << endl;
 #endif
     switch (fn.type)
     {
-        case formula_name_type::cell_reference:
+        case formula_name_t::cell_reference:
             m_formula_tokens.push_back(
                 make_unique<single_ref_token>(
                     address_t(
                         fn.address.sheet, fn.address.row, fn.address.col,
                         fn.address.abs_sheet, fn.address.abs_row, fn.address.abs_col)));
         break;
-        case formula_name_type::range_reference:
+        case formula_name_t::range_reference:
         {
             address_t first(fn.range.first.sheet, fn.range.first.row, fn.range.first.col,
                             fn.range.first.abs_sheet, fn.range.first.abs_row, fn.range.first.abs_col);
@@ -265,7 +266,7 @@ void formula_parser::name(const lexer_token_base& t)
             m_formula_tokens.push_back(make_unique<range_ref_token>(range_t(first, last)));
         }
         break;
-        case formula_name_type::table_reference:
+        case formula_name_t::table_reference:
         {
             table_t table;
             table.name = m_context.add_string(fn.table.name, fn.table.name_length);
@@ -275,11 +276,11 @@ void formula_parser::name(const lexer_token_base& t)
             m_formula_tokens.push_back(make_unique<table_ref_token>(table));
         }
         break;
-        case formula_name_type::function:
+        case formula_name_t::function:
             m_formula_tokens.push_back(
                 make_unique<function_token>(static_cast<size_t>(fn.func_oc)));
         break;
-        case formula_name_type::named_expression:
+        case formula_name_t::named_expression:
             m_formula_tokens.push_back(
                 make_unique<named_exp_token>(name.get(), name.size()));
         break;
@@ -312,10 +313,10 @@ void formula_parser::less(const lexer_token_base& t)
         next();
         switch (get_token().get_opcode())
         {
-            case op_equal:
+            case lexer_opcode_t::equal:
                 m_formula_tokens.push_back(make_unique<opcode_token>(fop_less_equal));
                 return;
-            case op_greater:
+            case lexer_opcode_t::greater:
                 m_formula_tokens.push_back(make_unique<opcode_token>(fop_not_equal));
                 return;
             default:
@@ -331,7 +332,7 @@ void formula_parser::greater(const lexer_token_base& t)
     if (has_next())
     {
         next();
-        if (get_token().get_opcode() == op_equal)
+        if (get_token().get_opcode() == lexer_opcode_t::equal)
         {
             m_formula_tokens.push_back(make_unique<opcode_token>(fop_greater_equal));
             return;
