@@ -25,8 +25,6 @@
 #include <sys/time.h>
 #endif
 
-#include <boost/thread/mutex.hpp>
-
 #define IXION_DEBUG_GLOBAL 0
 
 using namespace std;
@@ -54,15 +52,6 @@ double global::get_current_time()
 #endif
 }
 
-void global::sleep(unsigned int mseconds)
-{
-#ifdef _WIN32
-    ::Sleep(mseconds);
-#else
-    ::usleep(1000*mseconds);
-#endif
-}
-
 void global::load_file_content(const string& filepath, string& content)
 {
     ifstream file(filepath.c_str());
@@ -136,22 +125,6 @@ double global::to_double(const char* p, size_t n)
     return sign*val;
 }
 
-const char* get_formula_error_name(formula_error_t fe)
-{
-    static const char* default_err_name = "#ERR!";
-    static const char* names[] = {
-        "",        // no error
-        "#REF!",   // result not available
-        "#DIV/0!", // division by zero
-        "#NUM!"    // invalid expression
-    };
-    static const size_t name_size = 4;
-    if (static_cast<size_t>(fe) < name_size)
-        return names[fe];
-
-    return default_err_name;
-}
-
 // ============================================================================
 
 formula_error::formula_error(formula_error_t fe) :
