@@ -135,7 +135,7 @@ bool set_shared_formula_tokens_to_cell(
 
 }
 
-class model_context_impl : boost::noncopyable
+class model_context_impl
 {
     typedef std::map<std::string, unique_ptr<formula_cell>> named_expressions_type;
     typedef std::vector<std::string> strings_type;
@@ -147,12 +147,16 @@ class model_context_impl : boost::noncopyable
     typedef model_context::shared_tokens_type shared_tokens_type;
 
 public:
+    model_context_impl() = delete;
+    model_context_impl(const model_context_impl&) = delete;
+    model_context_impl& operator= (model_context_impl) = delete;
+
     model_context_impl(model_context& parent) :
         m_parent(parent),
         mp_config(new config),
         mp_cell_listener_tracker(new cell_listener_tracker(parent)),
-        mp_session_handler(NULL),
-        mp_table_handler(NULL)
+        mp_table_handler(NULL),
+        m_session_factory(ixion::make_unique<model_context::session_handler_factory>())
     {
     }
 
@@ -175,14 +179,14 @@ public:
         return *mp_cell_listener_tracker;
     }
 
-    iface::session_handler* get_session_handler()
+    std::unique_ptr<iface::session_handler> create_session_handler()
     {
-        return mp_session_handler;
+        return m_session_factory->create();
     }
 
-    void set_session_handler(iface::session_handler* handler)
+    void set_session_handler_factory(std::unique_ptr<model_context::session_handler_factory>&& factory)
     {
-        mp_session_handler = handler;
+        m_session_factory = std::move(factory);
     }
 
     iface::table_handler* get_table_handler()
@@ -265,7 +269,7 @@ public:
 
     double count_range(const abs_range_t& range, const values_t& values_type) const;
 
-    void get_all_formula_cells(dirty_formula_cells_t& cells) const;
+    dirty_formula_cells_t get_all_formula_cells() const;
 
 private:
     model_context& m_parent;
@@ -274,10 +278,11 @@ private:
 
     config* mp_config;
     cell_listener_tracker* mp_cell_listener_tracker;
-    iface::session_handler* mp_session_handler;
     iface::table_handler* mp_table_handler;
     named_expressions_type m_named_expressions;
 
+    std::unique_ptr<model_context::session_handler_factory> m_session_factory;
+
     formula_tokens_store_type m_tokens;
     model_context::shared_tokens_type m_shared_tokens;
     strings_type m_sheet_names; ///< index to sheet name map.
@@ -488,17 +493,17 @@ void model_context_impl::set_shared_formula(
         const char* p_formula, size_t n_formula, const char* p_range, size_t n_range,
         const formula_name_resolver& resolver)
 {
-    formula_name_type name_type = resolver.resolve(p_range, n_range, abs_address_t());
+    formula_name_t name_type = resolver.resolve(p_range, n_range, abs_address_t());
     abs_range_t range;
     switch (name_type.type)
     {
-        case ixion::formula_name_type::cell_reference:
+        case ixion::formula_name_t::cell_reference:
             range.first.sheet = name_type.address.sheet;
             range.first.row = name_type.address.row;
             range.first.column = name_type.address.col;
             range.last = range.first;
         break;
-        case ixion::formula_name_type::range_reference:
+        case ixion::formula_name_t::range_reference:
             range.first.sheet = name_type.range.first.sheet;
             range.first.row = name_type.range.first.row;
             range.first.column = name_type.range.first.col;
@@ -608,20 +613,18 @@ double count_formula_block(
     for (; pp != pp_end; ++pp)
     {
         const formula_cell& fc = **pp;
-        const formula_result* res = fc.get_result_cache();
-        if (!res)
-            continue;
+        const formula_result& res = fc.get_result_cache();
 
-        switch (res->get_type())
+        switch (res.get_type())
         {
-            case formula_result::rt_value:
+            case formula_result::result_type::value:
                 if (vt.is_numeric())
                     ++ret;
             break;
-            case formula_result::rt_string:
+            case formula_result::result_type::string:
                 if (vt.is_string())
                     ++ret;
-            case formula_result::rt_error:
+            case formula_result::result_type::error:
                 // TODO : how do we handle error formula cells?
             break;
         }
@@ -639,7 +642,7 @@ double model_context_impl::count_range(const abs_range_t& range, const values_t&
 
     double ret = 0.0;
     sheet_t last_sheet = range.last.sheet;
-    if (last_sheet >= m_sheets.size())
+    if (static_cast<size_t>(last_sheet) >= m_sheets.size())
         last_sheet = m_sheets.size() - 1;
 
     for (sheet_t sheet = range.first.sheet; sheet <= last_sheet; ++sheet)
@@ -704,8 +707,10 @@ double model_context_impl::count_range(const abs_range_t& range, const values_t&
     return ret;
 }
 
-void model_context_impl::get_all_formula_cells(dirty_formula_cells_t& cells) const
+dirty_formula_cells_t model_context_impl::get_all_formula_cells() const
 {
+    dirty_formula_cells_t cells;
+
     for (size_t sid = 0; sid < m_sheets.size(); ++sid)
     {
         const worksheet& sh = m_sheets[sid];
@@ -726,6 +731,8 @@ void model_context_impl::get_all_formula_cells(dirty_formula_cells_t& cells) con
             }
         }
     }
+
+    return cells;
 }
 
 void model_context_impl::erase_cell(const abs_address_t& addr)
@@ -995,16 +1002,20 @@ string_id_t model_context_impl::get_string_identifier_nowait(const abs_address_t
         case ixion::element_type_formula:
         {
             const formula_cell* p = col_store.get<formula_cell*>(addr.row);
-            const formula_result* res_cache = p->get_result_cache();
+            const formula_result* res_cache = p->get_result_cache_nowait();
+            if (!res_cache)
+                break;
+
             switch (res_cache->get_type())
             {
-                case formula_result::rt_string:
+                case formula_result::result_type::string:
                     return res_cache->get_string();
-                case formula_result::rt_error:
+                case formula_result::result_type::error:
                     // TODO : perhaps we should return the error string here.
                 default:
                     ;
             }
+            break;
         }
         default:
             ;
@@ -1036,6 +1047,11 @@ formula_cell* model_context_impl::get_formula_cell(const abs_address_t& addr)
     return col_store.get<formula_cell*>(addr.row);
 }
 
+std::unique_ptr<iface::session_handler> model_context::session_handler_factory::create()
+{
+    return std::unique_ptr<iface::session_handler>();
+}
+
 model_context::shared_tokens::shared_tokens() : tokens(NULL) {}
 model_context::shared_tokens::shared_tokens(formula_tokens_t* _tokens) : tokens(_tokens) {}
 model_context::shared_tokens::shared_tokens(const shared_tokens& r) : tokens(r.tokens), range(r.range) {}
@@ -1179,9 +1195,9 @@ matrix model_context::get_range_value(const abs_range_t& range) const
     return ret;
 }
 
-iface::session_handler* model_context::get_session_handler()
+std::unique_ptr<iface::session_handler> model_context::create_session_handler()
 {
-    return mp_impl->get_session_handler();
+    return mp_impl->create_session_handler();
 }
 
 iface::table_handler* model_context::get_table_handler()
@@ -1295,9 +1311,9 @@ sheet_t model_context::append_sheet(const char* p, size_t n, row_t row_size, col
     return mp_impl->append_sheet(p, n, row_size, col_size);
 }
 
-void model_context::set_session_handler(iface::session_handler* handler)
+void model_context::set_session_handler_factory(std::unique_ptr<session_handler_factory>&& factory)
 {
-    mp_impl->set_session_handler(handler);
+    mp_impl->set_session_handler_factory(std::move(factory));
 }
 
 void model_context::set_table_handler(iface::table_handler* handler)
@@ -1325,9 +1341,9 @@ const column_stores_t* model_context::get_columns(sheet_t sheet) const
     return mp_impl->get_columns(sheet);
 }
 
-void model_context::get_all_formula_cells(dirty_formula_cells_t& cells) const
+dirty_formula_cells_t model_context::get_all_formula_cells() const
 {
-    mp_impl->get_all_formula_cells(cells);
+    return mp_impl->get_all_formula_cells();
 }
 
 }
