@@ -102,7 +102,6 @@ model_parser::check_error::check_error(const string& msg) :
 
 model_parser::model_parser(const string& filepath, size_t thread_count) :
     m_context(),
-    m_session_handler(m_context),
     m_table_handler(),
     mp_table_entry(nullptr),
     mp_name_resolver(formula_name_resolver::get(formula_name_resolver_t::excel_a1, &m_context)),
@@ -110,7 +109,7 @@ model_parser::model_parser(const string& filepath, size_t thread_count) :
     m_thread_count(thread_count),
     m_print_separator(true)
 {
-    m_context.set_session_handler(&m_session_handler);
+    m_context.set_session_handler_factory(ixion::make_unique<session_handler::factory>(m_context));
     m_context.set_table_handler(&m_table_handler);
     m_context.append_sheet(IXION_ASCII("sheet"), 1048576, 1024);
 }
@@ -174,6 +173,9 @@ void model_parser::parse()
             }
             else if (buf_com.equals("mode init"))
             {
+                cout << get_formula_result_output_separator() << endl
+                    << "initializing" << endl;
+
                 parse_mode = parse_mode_init;
                 m_print_separator = true;
             }
@@ -185,6 +187,9 @@ void model_parser::parse()
             }
             else if (buf_com.equals("mode edit"))
             {
+                cout << get_formula_result_output_separator() << endl
+                    << "editing" << endl;
+
                 parse_mode = parse_mode_edit;
                 m_dirty_cells.clear();
                 m_dirty_cell_addrs.clear();
@@ -277,8 +282,8 @@ void model_parser::parse_init(const char*& p)
         throw model_parser::parse_error("separator is missing");
     }
 
-    formula_name_type ret = mp_name_resolver->resolve(name.get(), name.size(), abs_address_t());
-    if (ret.type != formula_name_type::cell_reference)
+    formula_name_t ret = mp_name_resolver->resolve(name.get(), name.size(), abs_address_t());
+    if (ret.type != formula_name_t::cell_reference)
     {
         ostringstream os;
         os << "invalid cell name: " << name.str();
@@ -297,6 +302,12 @@ void model_parser::parse_init(const char*& p)
         return;
     }
 
+    if (m_print_separator)
+    {
+        m_print_separator = false;
+        cout << get_formula_result_output_separator() << endl;
+    }
+
     switch (content_type)
     {
         case ct_formula:
@@ -310,6 +321,7 @@ void model_parser::parse_init(const char*& p)
             assert(p);
             m_dirty_cells.insert(pos);
             register_formula_cell(m_context, pos);
+            cout << name.str() << ": (f) " << buf.str() << endl;
 #if DEBUG_MODEL_PARSER
             std::string s;
             print_formula_tokens(m_context, pos, *tokens, s);
@@ -323,11 +335,6 @@ void model_parser::parse_init(const char*& p)
             __IXION_DEBUG_OUT__ << "pos: " << resolver.get_name(pos, false) << " type: string" << endl;
 #endif
             m_context.set_string_cell(pos, buf.get(), buf.size());
-            if (m_print_separator)
-            {
-                m_print_separator = false;
-                cout << get_formula_result_output_separator() << endl;
-            }
             cout << name.str() << ": (s) " << buf.str() << endl;
         }
         break;
@@ -339,12 +346,6 @@ void model_parser::parse_init(const char*& p)
             double value = global::to_double(buf.get(), buf.size());
             m_context.set_numeric_cell(pos, value);
 
-            if (m_print_separator)
-            {
-                m_print_separator = false;
-                cout << get_formula_result_output_separator() << endl;
-            }
-
             address_t pos_display(pos);
             pos_display.set_absolute(false);
             cout << mp_name_resolver->get_name(pos_display, abs_address_t(), false) << ": (n) " << value << endl;
@@ -446,8 +447,8 @@ void model_parser::parse_table(const char*& p)
             return;
 
         abs_address_t pos(0,0,0);
-        formula_name_type ret = mp_name_resolver->resolve(value.get(), value.size(), pos);
-        if (ret.type != formula_name_type::range_reference)
+        formula_name_t ret = mp_name_resolver->resolve(value.get(), value.size(), pos);
+        if (ret.type != formula_name_t::range_reference)
             throw parse_error("range of a table is expected to be given as a range reference.");
 
         entry.range = to_range(ret.range).to_abs(pos);
@@ -535,12 +536,12 @@ void model_parser::check()
         const formula_result& res = itr->second;
         cout << name << " : " << res.str(m_context) << endl;
 
-        formula_name_type name_type = mp_name_resolver->resolve(&name[0], name.size(), abs_address_t());
+        formula_name_t name_type = mp_name_resolver->resolve(&name[0], name.size(), abs_address_t());
         switch (name_type.type)
         {
-            case formula_name_type::cell_reference:
+            case formula_name_t::cell_reference:
             {
-                const formula_name_type::address_type& _addr = name_type.address;
+                const formula_name_t::address_type& _addr = name_type.address;
                 abs_address_t addr(_addr.sheet, _addr.row, _addr.col);
 
                 switch (m_context.get_celltype(addr))
@@ -548,14 +549,12 @@ void model_parser::check()
                     case celltype_t::formula:
                     {
                         const formula_cell* fcell = m_context.get_formula_cell(addr);
-                        const formula_result* res_cell = fcell->get_result_cache();
-                        if (!res_cell)
-                            throw check_error("result is not cached");
+                        const formula_result& res_cell = fcell->get_result_cache();
 
-                        if (*res_cell != res)
+                        if (res_cell != res)
                         {
                             ostringstream os;
-                            os << "unexpected result: (expected: " << res.str(m_context) << "; actual: " << res_cell->str(m_context) << ")";
+                            os << "unexpected result: (expected: " << res.str(m_context) << "; actual: " << res_cell.str(m_context) << ")";
                             throw check_error(os.str());
                         }
                     }
@@ -592,17 +591,15 @@ void model_parser::check()
                 }
             }
             break;
-            case formula_name_type::named_expression:
+            case formula_name_t::named_expression:
             {
                 const formula_cell* fcell = m_context.get_named_expression(name);
-                const formula_result* res_cell = fcell->get_result_cache();
-                if (!res_cell)
-                    throw check_error("result is not cached");
+                const formula_result& res_cell = fcell->get_result_cache();
 
-                if (*res_cell != res)
+                if (res_cell != res)
                 {
                     ostringstream os;
-                    os << "unexpected result: (expected: " << res.str(m_context) << "; actual: " << res_cell->str(m_context) << ")";
+                    os << "unexpected result: (expected: " << res.str(m_context) << "; actual: " << res_cell.str(m_context) << ")";
                     throw check_error(os.str());
                 }
             }
