@@ -5,23 +5,27 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#ifndef __IXION_SESSION_HANDLER_HPP__
-#define __IXION_SESSION_HANDLER_HPP__
+#ifndef INCLUDED_IXION_SESSION_HANDLER_HPP
+#define INCLUDED_IXION_SESSION_HANDLER_HPP
 
 #include "ixion/interface/session_handler.hpp"
 #include "ixion/model_context.hpp"
 
-#include <string>
+#include <memory>
 
 namespace ixion {
 
 class session_handler : public iface::session_handler
 {
+    struct impl;
+    std::unique_ptr<impl> mp_impl;
+
 public:
     session_handler(const model_context& cxt);
     virtual ~session_handler();
 
     virtual void begin_cell_interpret(const abs_address_t& pos);
+    virtual void end_cell_interpret();
     virtual void set_result(const formula_result& result);
     virtual void set_invalid_expression(const char* msg);
     virtual void set_formula_error(const char* msg);
@@ -34,13 +38,25 @@ public:
     virtual void push_table_ref(const table_t& table);
     virtual void push_function(formula_function_t foc);
 
-private:
-    const model_context& m_context;
-    std::unique_ptr<formula_name_resolver> mp_resolver;
-    std::string m_cell_name;
+    class factory : public model_context::session_handler_factory
+    {
+        const model_context& m_context;
+    public:
+        factory(const model_context& cxt);
+
+        virtual std::unique_ptr<iface::session_handler> create();
+    };
+
+    /**
+     * Print string to stdout in a thread-safe way.
+     *
+     * @param msg string to print to stdout.
+     */
+    static void print(const std::string& msg);
 };
 
 }
 
 #endif
+
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
